@@ -27,6 +27,7 @@
 
 /* most cards in a single system */
 #define NV_MAX_DEVICES 32
+#define GPU_UUID_LEN    (16)
 
 typedef struct {
     NvU32    domain;        /* PCI domain number   */
@@ -323,6 +324,11 @@ typedef struct
     NvU16 subsystem_id;
     NvU16 subsystem_vendor;
     NvU32 gpu_id;
+    struct
+    {
+        NvBool         bValid;
+        NvU8           uuid[GPU_UUID_LEN];
+    } nv_gpu_uuid_cache;
     void *handle;
 
     NvU32 pci_cfg_space[NVRM_PCICFG_NUM_DWORDS];
@@ -353,22 +359,18 @@ typedef struct
 } nv_state_t;
 
 // Forward define the gpu ops structures
-typedef struct gpuSession      *nvgpuSessionHandle_t;
-typedef struct gpuAddressSpace *nvgpuAddressSpaceHandle_t;
-typedef struct gpuChannel      *nvgpuChannelHandle_t;
-typedef struct gpuObject       *nvgpuObjectHandle_t;
-typedef struct gpuChannelInfo  *nvgpuChannelInfo_t;
-typedef struct gpuCaps         *nvgpuCaps_t;
-typedef struct gpuAllocInfo    *nvgpuAllocInfo_t;
-
-typedef struct
-{
-    /* PCI config info */
-    nv_pci_info_t pci_info;
-    void *handle;
-    nv_aperture_t bar0;
-    nv_aperture_t *regs;
-} nv_smu_state_t;
+typedef struct gpuSession       *nvgpuSessionHandle_t;
+typedef struct gpuAddressSpace  *nvgpuAddressSpaceHandle_t;
+typedef struct gpuChannel       *nvgpuChannelHandle_t;
+typedef struct gpuObject        *nvgpuObjectHandle_t;
+typedef struct gpuChannelInfo   *nvgpuChannelInfo_t;
+typedef struct gpuCaps          *nvgpuCaps_t;
+typedef struct gpuAllocInfo     *nvgpuAllocInfo_t;
+typedef struct gpuVaAllocInfo   *nvgpuVaAllocInfo_t;
+typedef struct gpuMapInfo       *nvgpuMapInfo_t;
+typedef struct gpuFbInfo        *nvgpuFbInfo_t;
+typedef struct gpuFaultInfo     *nvgpuFaultInfo_t;
+typedef struct gpuInfo          *nvgpuInfo_t;
 
 /*
  * flags
@@ -578,9 +580,18 @@ NV_STATUS  NV_API_CALL  nv_alias_pages           (nv_state_t *, NvU32, NvU32, Nv
 NV_STATUS  NV_API_CALL  nv_alloc_pages           (nv_state_t *, NvU32, NvBool, NvU32, NvBool, NvU64 *, void **);
 NV_STATUS  NV_API_CALL  nv_free_pages            (nv_state_t *, NvU32, NvBool, NvU32, void *);
 
-NV_STATUS  NV_API_CALL  nv_dma_map_pages         (nv_state_t *, NvU64, NvU64 *, void **);
+NV_STATUS  NV_API_CALL  nv_register_user_pages   (nv_state_t *, NvU64, NvU64 *, void **);
+NV_STATUS  NV_API_CALL  nv_unregister_user_pages (nv_state_t *, NvU64, void **);
+
+NV_STATUS  NV_API_CALL  nv_dma_map_pages         (nv_state_t *, NvU64, NvU64 *, NvBool, void **);
 NV_STATUS  NV_API_CALL  nv_dma_unmap_pages       (nv_state_t *, NvU64, NvU64 *, void **);
 
+NV_STATUS  NV_API_CALL  nv_dma_map_alloc         (nv_state_t *, NvU64, NvU64 *, void **);
+NV_STATUS  NV_API_CALL  nv_dma_unmap_alloc       (nv_state_t *, NvU64, NvU64 *, void **);
+
+NV_STATUS  NV_API_CALL  nv_dma_map_peer          (nv_state_t *, nv_aperture_t *, NvU64, NvU64 *);
+NV_STATUS  NV_API_CALL  nv_dma_unmap_peer        (nv_state_t *, NvU64, NvU64 *);
+
 NvS32  NV_API_CALL  nv_start_rc_timer            (nv_state_t *);
 NvS32  NV_API_CALL  nv_stop_rc_timer             (nv_state_t *);
 
@@ -601,23 +612,14 @@ NV_STATUS  NV_API_CALL  nv_acpi_method           (NvU32, NvU32, NvU32, void *, N
 NV_STATUS  NV_API_CALL  nv_acpi_dsm_method       (nv_state_t *, NvU8 *, NvU32, NvU32, void *, NvU16, NvU32 *, void *, NvU16 *);
 NV_STATUS  NV_API_CALL  nv_acpi_ddc_method       (nv_state_t *, void *, NvU32 *);
 NV_STATUS  NV_API_CALL  nv_acpi_dod_method       (nv_state_t *, NvU32 *, NvU32 *);
-void*      NV_API_CALL  nv_get_smu_state         (void);
 NV_STATUS  NV_API_CALL  nv_acpi_rom_method       (nv_state_t *, NvU32 *, NvU32 *);
 NV_STATUS  NV_API_CALL  nv_log_error             (nv_state_t *, NvU32, const char *, va_list);
 
-#if defined(NV_MOBILE_DGPU)
-NV_STATUS  NV_API_CALL  nv_nvmap_create_client   (void);
-void       NV_API_CALL  nv_nvmap_destroy_client  (void);
-void       NV_API_CALL  nv_nvmap_free_handle_id  (NvU32);
-NV_STATUS  NV_API_CALL  nv_nvmap_dup_handle_id   (NvU32 *, NvU32);
-NV_STATUS  NV_API_CALL  nv_nvmap_get_page_list_info (NvU32, NvU32 *, NvU32 *, NvU32 *, BOOL *, NvU32 *);
-NV_STATUS  NV_API_CALL  nv_nvmap_get_page_list   (nv_state_t *, void *, NvU64, NvU64 *, void **, NvU32);
-NV_STATUS  NV_API_CALL  nv_nvmap_release_pages   (nv_state_t *, NvU64, NvU64 *, void *, NvU32);
-#endif
 NV_STATUS  NV_API_CALL  nv_alloc_os_descriptor_handle (nv_state_t *, NvS32, void *, NvU64, NvU32 *);
 NvU64      NV_API_CALL  nv_get_dma_start_address (nv_state_t *);
 NV_STATUS  NV_API_CALL  nv_set_primary_vga_status(nv_state_t *);
 NV_STATUS  NV_API_CALL  nv_pci_trigger_recovery  (nv_state_t *);
+NvBool     NV_API_CALL  nv_requires_dma_remap    (nv_state_t *);
 
 #if defined(NVCPU_X86) || defined(NVCPU_X86_64)
 NvBool     NV_API_CALL  nv_is_virtualized_system  (nv_stack_t *);
@@ -683,15 +685,11 @@ NV_STATUS  NV_API_CALL  rm_i2c_transfer           (nvidia_stack_t *, nv_state_t
 NV_STATUS  NV_API_CALL  rm_perform_version_check  (nvidia_stack_t *, void *, NvU32);
 
 NV_STATUS  NV_API_CALL  rm_system_event           (nvidia_stack_t *, NvU32, NvU32);
-NV_STATUS  NV_API_CALL  rm_init_smu               (nvidia_stack_t *, nv_smu_state_t *);
-NV_STATUS  NV_API_CALL  rm_shutdown_smu           (nvidia_stack_t *, nv_smu_state_t *);
-NV_STATUS  NV_API_CALL  rm_suspend_smu            (nvidia_stack_t *);
-NV_STATUS  NV_API_CALL  rm_resume_smu             (nvidia_stack_t *);
 
 void       NV_API_CALL  rm_disable_gpu_state_persistence    (nvidia_stack_t *sp, nv_state_t *);
 NV_STATUS  NV_API_CALL  rm_p2p_init_mapping       (nvidia_stack_t *, NvU64, NvU64 *, NvU64 *, NvU64 *, NvU64 *, NvU64, NvU64, NvU64, NvU64, void (*)(void *), void *);
 NV_STATUS  NV_API_CALL  rm_p2p_destroy_mapping    (nvidia_stack_t *, NvU64);
-NV_STATUS  NV_API_CALL  rm_p2p_get_pages          (nvidia_stack_t *, NvU64, NvU32, NvU64, NvU64, NvU64 **, NvU32 **, NvU32 **, NvU32 *, void *, void (*)(void *), void *);
+NV_STATUS  NV_API_CALL  rm_p2p_get_pages          (nvidia_stack_t *, NvU64, NvU32, NvU64, NvU64, NvU64 *, NvU32 *, NvU32 *, NvU32 *, void *, void (*)(void *), void *);
 NV_STATUS  NV_API_CALL  rm_p2p_put_pages          (nvidia_stack_t *, NvU64, NvU32, NvU64, void *);
 NV_STATUS  NV_API_CALL  rm_get_cpu_type           (nvidia_stack_t *, nv_cpu_type_t *);
 NV_STATUS  NV_API_CALL  rm_log_gpu_crash          (nv_stack_t *, nv_state_t *);
@@ -702,22 +700,35 @@ NV_STATUS  NV_API_CALL  rm_gpu_ops_address_space_create(nvidia_stack_t *, nvgpuS
 NV_STATUS  NV_API_CALL  rm_gpu_ops_address_space_create_mirrored(nvidia_stack_t *, nvgpuSessionHandle_t, unsigned long long, unsigned long long, nvgpuAddressSpaceHandle_t *);
 NV_STATUS  NV_API_CALL  rm_gpu_ops_address_space_destroy(nvidia_stack_t *, nvgpuAddressSpaceHandle_t);
 NV_STATUS  NV_API_CALL  rm_gpu_ops_memory_alloc_fb(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvLength, NvU64 *, nvgpuAllocInfo_t);
+NV_STATUS  NV_API_CALL  rm_gpu_ops_memory_alloc_gpu_va(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvLength, NvU64 *, nvgpuVaAllocInfo_t);
+NV_STATUS  NV_API_CALL  rm_gpu_ops_memory_alloc_gpu_pa(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvLength, NvU64 *, nvgpuAllocInfo_t);
+
 NV_STATUS  NV_API_CALL  rm_gpu_ops_memory_alloc_sys(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvLength, NvU64 *, nvgpuAllocInfo_t);
-NV_STATUS  NV_API_CALL  rm_gpu_ops_memory_cpu_map(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU64, NvLength, void **);
+NV_STATUS  NV_API_CALL  rm_gpu_ops_memory_cpu_map(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU64, NvLength, void **, NvU32);
 NV_STATUS  NV_API_CALL  rm_gpu_ops_memory_cpu_ummap(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, void*);
 NV_STATUS  NV_API_CALL  rm_gpu_ops_channel_allocate(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, nvgpuChannelHandle_t *, nvgpuChannelInfo_t);
 NV_STATUS  NV_API_CALL  rm_gpu_ops_channel_destroy(nvidia_stack_t *, nvgpuChannelHandle_t);
 const char*  NV_API_CALL  rm_gpu_ops_channel_translate_error(nvidia_stack_t *, NvU32);
 NV_STATUS  NV_API_CALL  rm_gpu_ops_copy_engine_allocate(nvidia_stack_t *, nvgpuChannelHandle_t, NvU32, NvU32*, nvgpuObjectHandle_t *);
 NV_STATUS  NV_API_CALL rm_gpu_ops_memory_free(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU64);
+NV_STATUS  NV_API_CALL rm_gpu_ops_memory_free_va(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU64);
+NV_STATUS  NV_API_CALL rm_gpu_ops_memory_free_pa(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU64);
 NV_STATUS  NV_API_CALL rm_gpu_ops_query_caps(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, nvgpuCaps_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_attached_uuids(nvidia_stack_t *, NvU8 *pUuidList, unsigned *numGpus);
-NV_STATUS  NV_API_CALL rm_gpu_ops_get_gpu_arch(nvidia_stack_t *, NvU8 *pUuid, unsigned uuidLength, NvU32 *pGpuArch);
+NV_STATUS  NV_API_CALL rm_gpu_ops_get_gpu_info(nvidia_stack_t *, NvU8 *pUuid, unsigned uuidLength, nvgpuInfo_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_uvm_priv_region(NvU64 *pPrivRegionStart, NvU64 *pPrivRegionLength);
 NV_STATUS  NV_API_CALL rm_gpu_ops_service_device_interrupts_rm(nvidia_stack_t *, nvgpuChannelHandle_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_check_ecc_error_slowpath(nvidia_stack_t *, nvgpuChannelHandle_t, NvBool *);
 NV_STATUS  NV_API_CALL rm_gpu_ops_kill_channel(nvidia_stack_t *, nvgpuChannelHandle_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_gmmu_fmt(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, void **);
+NV_STATUS  NV_API_CALL rm_gpu_ops_dup_allocation(nvidia_stack_t *, NvHandle, nvgpuAddressSpaceHandle_t, NvU64, nvgpuAddressSpaceHandle_t, NvU64*, NvBool);
+NV_STATUS  NV_API_CALL rm_gpu_ops_free_duped_handle(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvHandle);
+NV_STATUS  NV_API_CALL rm_gpu_ops_get_fb_info(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, nvgpuFbInfo_t);
+NV_STATUS  NV_API_CALL rm_gpu_ops_get_gpu_ids(nvidia_stack_t *sp, NvU8 *pUuid, unsigned uuidLength, NvU32 *pDeviceId, NvU32 *pSubdeviceId);
+NV_STATUS  NV_API_CALL rm_gpu_ops_init_fault_info(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, nvgpuFaultInfo_t);
+NV_STATUS  NV_API_CALL rm_gpu_ops_destroy_fault_info(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, nvgpuFaultInfo_t);
+
+void       NV_API_CALL rm_kernel_rmapi_op(nvidia_stack_t *sp, void *ops_cmd);
 
 #endif /* NVWATCH */
 
