@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 1993-2011 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 1993-2014 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -27,11 +27,6 @@
 #define RM_OK                           NV_OK           //STATUS_SUCCESS
 #define RM_ERROR                        NV_ERR_GENERIC  //STATUS_FAILURE
 
-/*
-  This bit is reserved, no RM_STATUS should ever have it set
-  NVOS_STATUS_IN_RM_STATUS              0x80000000
-*/
-
 /*!
  * @def         RM_STATUS_LEVEL_OK
  * @see         RM_STATUS_LEVEL
